@@ -121,12 +121,24 @@ typedef struct {
 	std::string write_prefix;
 } cooling_dev_t;
 
+typedef struct {
+	std::string name;
+	int valid;
+	int power_limit_min;
+	int power_limit_max;
+	int time_wind_min;
+	int time_wind_max;
+	int step_size;
+} ppcc_t;
+
 typedef struct {
 	std::string name;
 	std::string uuid;
 	std::string product_name;
+	std::string product_sku;
 	int default_preference;
 	int polling_interval;
+	ppcc_t ppcc;
 	std::vector<thermal_sensor_t> sensors;
 	std::vector<thermal_zone_t> zones;
 	std::vector<cooling_dev_t> cooling_devs;
@@ -141,6 +153,7 @@ private:
 	int matched_thermal_info_index;
 	xmlDoc *doc;
 	xmlNode *root_element;
+	int auto_config;
 
 	int parse(xmlNode * a_node, xmlDoc *doc);
 	int parse_pid_values(xmlNode * a_node, xmlDoc *doc, pid_control_t *pid_ptr);
@@ -171,8 +184,11 @@ private:
 			thermal_zone_t *info_ptr);
 	int parse_new_platform(xmlNode * a_node, xmlDoc *doc, thermal_info_t *info);
 
+	int parse_ppcc(xmlNode * a_node, xmlDoc *doc, ppcc_t *ppcc);
+
 	void string_trim(std::string &str);
 	char *char_trim(char *trim);
+	bool match_product_sku(int index);
 
 public:
 	cthd_parse();
@@ -182,6 +198,7 @@ public:
 	void dump_thermal_conf();
 	bool platform_matched();
 	int get_polling_interval();
+	ppcc_t *get_ppcc_param(std::string name);
 	int zone_count() {
 		return thermal_info_list[matched_thermal_info_index].zones.size();
 	}
@@ -193,6 +210,14 @@ public:
 		return thermal_info_list[matched_thermal_info_index].sensors.size();
 	}
 
+	int thermal_conf_auto() {
+		return auto_config;
+	}
+
+	int thermal_matched_platform_index() {
+		return matched_thermal_info_index;
+	}
+
 	int set_default_preference();
 	int trip_count(unsigned int zone_index);
 	bool pid_status(int cdev_index);
